import java.awt.event.*;
import java.awt.*;
import java.util.*;

public class CBunker extends CKillable
{
  private Vector aim;
  private double turnMomentum;
  private ArrayList weapons;
  private int currWeapon;
  private int energy;
  private boolean shielded;
  private double currPower;
  private int energyCounter;
  private int MAX_ENERGY;
  private int MAX_HEALTH;
  private int score;
  private int money;
  
  public static final int MGUN_WEAPON = 0;
  public static final int ROCKET_WEAPON = 1;
  public static final int FLAK_WEAPON= 2;
  public static final int LASER_WEAPON = 3;
  
  private boolean bHasMGun;
  private boolean bHasRockets;
  private boolean bHasFlak;
  private boolean bHasLasers;
  
  public static final int WIDTH = 100;
  public static final int HEIGHT = 120;    // Hitbox extends to underground...
  
  public CBunker(CWorld world, Vector pos, Vector vel, Vector acc)
  {
    super(world, pos, vel, acc);
    hitBox = new Rectangle((int)pos.x()-WIDTH/2, (int)pos.y()-HEIGHT/2, WIDTH, HEIGHT);
    aim.setLength(currPower);
    leftImg = getImage("bunker.png");
    bHasMGun = true;
    bHasRockets = bHasFlak = bHasLasers = false;
    currWeapon = 0;
  }
    
  public void turnLeft()
  {
    if(aim.angle() < 3.0*Math.PI/4.0) aim.rotate(0.04);      
  }
  
  public void turnRight()
  {
    if(aim.angle() > Math.PI/4.0) aim.rotate(-0.04);
  }
  
  public void raisePower()
  {
    if(currPower > 25) return;
    currPower += 0.3;
    aim.setLength(currPower);
  }
  
  public void lowerPower()
  {
    if(currPower < 10) return;
    currPower -= 0.3;
    aim.setLength(currPower);
  }
  
  public void setShields(boolean state)
  {
    if(energy <= 0) { shielded = false; return; }
    shielded = state;
  }
  
  public void setAim(Vector v)
  {
    if(aim.y() < 2 && aim.x() < 2) { return; }
    
    aim = v;
    
    if(aim.angle() < Math.PI/4.0)
      aim.setAngle(Math.PI/4.0);
    else if(aim.angle() > 3.0*Math.PI/4.0)
      aim.setAngle(3.0*Math.PI/4.0);
    if(aim.length() > 25.0)
      aim.setLength(25.0);
    if(aim.length() < 2)
      aim.setLength(2);
    
  }
  
  public void fire()
  {
    CWeapon myWeapon = (CWeapon) weapons.get(currWeapon);
    if(myWeapon instanceof CLaserWeapon && energy > 0) { energy--; }
    else if (myWeapon instanceof CLaserWeapon) { return; }
    if(myWeapon.getAmmo() == 0) { return; }
    myWeapon.fire(aim);
  }
  
  public void spawn(Vector pos, Vector vel, Vector acc)
  {
    this.pos = new Vector(pos);
    this.vel = new Vector();
    this.acc = new Vector();
    aim = new Vector(0, 7);
    currPower = 15.0;
    aim.setLength(currPower);
    weapons = new ArrayList();
    weapons.add(new CMGunWeapon(world, pos, new Vector(), new Vector()));
    weapons.add(new CRocketWeapon(world, pos, new Vector(), new Vector()));
    weapons.add(new CChaffWeapon(world, pos, new Vector(), new Vector()));
    weapons.add(new CLaserWeapon(world, pos, new Vector(), new Vector()));
    energy = 150;
    health = 100;
    shielded = false;
    energyCounter = 0;
    MAX_ENERGY = 150;
    MAX_HEALTH = 100;
    score = 0;
    money = 0;
    turnMomentum = 0.0;
  }
  
  public void draw(Graphics g)
  {
    Vector v = new Vector(pos);
    Vector a = new Vector(aim);
    a.scale(5);
    v.addVector(a);
    g.setColor(Color.RED);
    if(shielded) g.setColor(Color.GREEN);
    g.drawImage(leftImg, (int)pos.x()-WIDTH/2, (int)world.getHeight()-(int)pos.y()-HEIGHT/2, null);
    //g.fillArc((int)pos.x()-WIDTH/2, (int)world.getHeight()-(int)pos.y()-HEIGHT/2, WIDTH,HEIGHT,0,180);
    g.drawRect((int)v.x()-1, (int)(world.getHeight()-v.y())-1, 2, 2);  // Targeter
  //  g.drawRect(hitBox.x, (int)(world.getHeight()-hitBox.y-hitBox.height), hitBox.width, hitBox.height);
    
    // Health and energy bars
    g.setColor(Color.GREEN);
    g.drawRect(hitBox.x, (int)(world.getHeight()-10), hitBox.width, 5);
    g.fillRect(hitBox.x, (int)(world.getHeight()-10), (int)((double)hitBox.width*(health/100.0)), 5);
    
    g.setColor(Color.BLUE);
    g.drawRect(hitBox.x, (int)(world.getHeight()-5), hitBox.width, 5);
    g.fillRect(hitBox.x, (int)(world.getHeight()-5), (int)((double)hitBox.width*(energy/(double)MAX_ENERGY)), 5);
  }
  
  public void update()
  {
    super.update();
    ((CWeapon) weapons.get(currWeapon)).update();
    if(shielded) energy-=3;
    if(energy <= 0) setShields(false);
    if(energyCounter < 7) energyCounter++;
    else { energyCounter = 0; if(energy < MAX_ENERGY) energy+=4; }
  }
  
  public void takeDamage(int damage)
  {
    if(shielded) return;
    health -= damage;
    if(health <= 0)
      die();
  }
  
  public void setWeapon(int w)
  {
    if(w == MGUN_WEAPON && !bHasMGun) return;
    if(w == ROCKET_WEAPON && !bHasRockets) return;
    if(w == FLAK_WEAPON && !bHasFlak) return;
    if(w == LASER_WEAPON && !bHasLasers) return;
    currWeapon = w;
  }
  
  public boolean buyWeapon(int w)
  {
    if(w == ROCKET_WEAPON && !bHasRockets) { bHasRockets = true; return true; }
    if(w == FLAK_WEAPON && !bHasFlak) { bHasFlak = true; return true; }
    if(w == LASER_WEAPON && !bHasLasers) { bHasLasers = true; return true; }
    return false;
  }
  
  public boolean isDead()
  {
    if(health <= 0) return true;
    return false;
  }
  
  public void refillEnergy()
  {
    energy = MAX_ENERGY;
  }
  
  public void restoreHealth()
  {
    health = MAX_HEALTH;
  }
  
  public void addHealth(int dHealth)
  {
    health += dHealth;
    if(health > MAX_HEALTH) health = MAX_HEALTH;
  }
  
  public int getHealth()
  {
    return health;
  }
  public void addScore(int dScore)
  {
    score += dScore;
  }
    
  public void addMoney(int dMoney)
  {
    money += dMoney;
  }
  
  public int getScore()
  {
    return score;
  }
  
  public int getMoney()
  {
    return money;
  }
  
  public String getAmmo()
  {
    CWeapon w = (CWeapon)weapons.get(currWeapon);
    if(currWeapon == MGUN_WEAPON) return "Infinite";
    else if(currWeapon == ROCKET_WEAPON) return "" + w.getAmmo();
    else if(currWeapon == FLAK_WEAPON) return "" + w.getAmmo();
    else if(currWeapon == LASER_WEAPON) return "Infinite";
    
    return "???";
  }
  
  public void addRockets(int dRockets)
  {
    CWeapon w = (CWeapon)weapons.get(ROCKET_WEAPON);
    w.addAmmo(dRockets);
  }
   
  public void addMortars(int dMortars)
  {
    CWeapon w = (CWeapon)weapons.get(FLAK_WEAPON);
    w.addAmmo(dMortars);
  }
}