import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.URL;

public class CWorld extends JPanel implements KeyListener, ActionListener, MouseListener, MouseMotionListener
{
  public static final Vector GRAVITY = new Vector(0, -0.5);
  
  private ArrayList bunkers;
  private ArrayList bombs;
  private ArrayList bullets;
  private ArrayList explosions;
  private ArrayList planes;
  private ArrayList doodads;
  
  private ArrayList newThings;
  private ArrayList deadThings;
  
  private boolean kLeft;
  private boolean kRight;
  private boolean kSpace;
  private boolean kUp;
  private boolean kDown;
  private boolean kShift;
  
  private boolean buyTime;
  
  private Rectangle bounds;
  
  private CBuyMenu buyMenu;
  private Image bgImage;
  private BorderLayout bLayout;
  
  private int level;
  
  public CWorld()
  {
    this(new Dimension(800, 600));
  }
  
  public CWorld(Dimension size)
  {
    setPreferredSize(size);
    bLayout = new BorderLayout();
    setLayout(bLayout);
    bounds = new Rectangle((int)size.getWidth(), (int)size.getHeight());
    
    bunkers = new ArrayList();
    bombs = new ArrayList();
    bullets = new ArrayList();
    explosions = new ArrayList();
    planes = new ArrayList();
    doodads = new ArrayList();
    
    newThings = new ArrayList();
    deadThings = new ArrayList();
    
    
    addKeyListener(this);
    kLeft = kRight = kUp = kDown = kSpace = kShift = false;
    buyTime = false;
    
    addMouseListener(this);
    addMouseMotionListener(this);    
    bgImage = getImage("bg.png");
    
    level = 0;
    buyMenu = new CBuyMenu(this);
    add(Box.createVerticalStrut(100), BorderLayout.NORTH);
    add(Box.createVerticalStrut(100), BorderLayout.SOUTH);
    add(Box.createHorizontalStrut(100), BorderLayout.WEST);
    add(Box.createHorizontalStrut(100), BorderLayout.EAST);
    add(buyMenu, BorderLayout.CENTER);
  }
  
  public void newGame()
  {
    bunkers.clear();
    planes.clear();
    bullets.clear();
    explosions.clear();
    bombs.clear();
    doodads.clear();
    
    level = 1;
    initLevel(1);
    bunkers.add(new CBunker(this, new Vector(getWidth()/2, 0), new Vector(), new Vector()));
  }
  
  public void initLevel(int level)
  {
    Random numGen = new Random();
    
    for(int i=0; i<level; i++)
    {
      planes.add(new CPlane(this, new Vector(0, numGen.nextInt(150)+150), new Vector(numGen.nextDouble()*5+1.0, 0), new Vector()));
      planes.add(new CPlane(this, new Vector(0, numGen.nextInt(150)+350), new Vector(numGen.nextDouble()*5+1.0, 0), new Vector()));
      planes.add(new CPlane(this, new Vector(getWidth(), numGen.nextInt(150)+400), new Vector(-5*numGen.nextDouble()-1.0, 0), new Vector()));
    }
    if((level+5) % 5 == 0)
    {
      planes.add(new CBoss(this, new Vector(100, 500), new Vector(2, 0), new Vector(0, 0), level));
    }
  }
  
  public boolean levelIsOver()
  {
    return (bombs.isEmpty() && bullets.isEmpty() && explosions.isEmpty() && doodads.isEmpty()
              && planes.isEmpty());
  }
  
  public int getWidth()
  {
    return (int)bounds.getWidth();
  }
  public int getHeight()
  {
    return (int)bounds.getHeight();
  }
  
  public void addThing(CGameObject obj)
  {
    newThings.add(obj);
  }
  public void removeThing(CGameObject obj)
  {
    deadThings.add(obj);
  }
  
  void addNewThings()
  {
    Iterator it = newThings.iterator();
    while(it.hasNext())
    {
      CGameObject obj = (CGameObject)it.next();
      
      // Put object into appropriate ArrayList
      if(obj instanceof CBunker)
      {
        bunkers.add(obj);
      }
      else if(obj instanceof CBomb)
      {
        bombs.add(obj);
      }
      else if(obj instanceof CAmmo)
      {
        bullets.add(obj);
      }
      else if(obj instanceof CExplosion)
      {
        explosions.add(obj);
      }
      else if(obj instanceof CPlane)
      {
        planes.add(obj);
      }
      else if(obj instanceof CDoodad)
      {
        doodads.add(obj);
      }
    }
    
    newThings.clear();
  }
  
  void removeDeadThings()
  {
    Iterator it = deadThings.iterator();
    CBunker b = getBunker();
    while(it.hasNext())
    {
      CGameObject obj = (CGameObject)it.next();
      
      // Put object into appropriate ArrayList
      if(obj instanceof CBunker)
      {
        if(bunkers.indexOf(obj) != -1) bunkers.remove(bunkers.indexOf(obj));
      }
      else if(obj instanceof CBomb)
      {
        if(bombs.indexOf(obj) != -1) bombs.remove(bombs.indexOf(obj));
      }
      else if(obj instanceof CAmmo)
      {
        if(bullets.indexOf(obj) != -1) bullets.remove(bullets.indexOf(obj));
      }
      else if(obj instanceof CExplosion)
      {
        if(explosions.indexOf(obj) != -1) explosions.remove(explosions.indexOf(obj));
      }
      else if(obj instanceof CPlane)
      {
        if(planes.indexOf(obj) != -1) planes.remove(planes.indexOf(obj));
        if(obj instanceof CBoss) { b.addScore(900); b.addMoney(900); }
        b.addScore(100);
        b.addMoney(100);
      }
      else if(obj instanceof CDoodad)
      {
        if(doodads.indexOf(obj) != -1) doodads.remove(doodads.indexOf(obj));
      }
    }
    
    deadThings.clear();
  }
  
  public boolean outOfBounds(Vector pos)
  {
    if(pos.x() < bounds.getX() || pos.x() > bounds.getX() + bounds.getWidth() ||
       pos.y() < bounds.getY() || pos.y() > bounds.getY() + bounds.getHeight())
      return true;
    return false;
  }
  
  void updateThings()
  {
    checkHit();
    addNewThings();
    removeDeadThings();
    if(bunkers.size() > 0)
    {
      CBunker b = getBunker();
      if(kLeft)
      {
        b.turnLeft();
      }
      if(kRight)
      {
        b.turnRight();
      }
      if(kSpace)
      {
        b.fire();
      }
      if(kUp)
      {
        b.raisePower();
      }
      if(kDown)
      {
        b.lowerPower();
      }
      if(kShift)
      {
        b.setShields(true);
      }
      else
      {
        b.setShields(false);
      }
    }
    
    Iterator it = bunkers.iterator();
    while(it.hasNext())
    {
      CBunker b = (CBunker)it.next();
      b.update();
    }
    
    it = bullets.iterator();
    while(it.hasNext())
    {
      CAmmo b = (CAmmo)it.next();
      b.update();
    }
    
    it = bombs.iterator();
    while(it.hasNext())
    {
      CBomb b = (CBomb)it.next();
      b.update();
    }
    
    it = explosions.iterator();
    while(it.hasNext())
    {
      CExplosion e = (CExplosion)it.next();
      e.update();
    }
    
    it = doodads.iterator();
    while(it.hasNext())
    {
      CDoodad d = (CDoodad)it.next();
      d.update();
    }
    
    it = planes.iterator();
    while(it.hasNext())
    {
      CPlane p = (CPlane)it.next();
      p.update();
    }
    
    repaint();
  }
  
  public void paintComponent(Graphics g)
  {
    g.setColor(new Color(192, 192, 255));
    g.setColor(Color.BLACK);
    //g.fillRect(0, 0, (int)bounds.getWidth(), (int)bounds.getHeight());
    g.drawImage(bgImage, 0, 0, null);
    
    if(bunkers.size() != 0)
    {
      Iterator it = bullets.iterator();
      
      while(it.hasNext())
      {
        CAmmo b = (CAmmo)it.next();
        b.draw(g);
      }
      
      it = bunkers.iterator();
      while(it.hasNext())
      {
        CBunker b = (CBunker)it.next();
        b.draw(g);
        g.drawString("Score: "+b.getScore(), 20, 20);
        g.drawString("Money: "+b.getMoney(), 20, 40);
        g.drawString("Ammo left: "+ b.getAmmo(), 20, 60);
      }
      g.drawString("Level " + level, 950, 20);
      
      try
      {
        it = bombs.iterator();
        while(it.hasNext())
        {
          CBomb b = (CBomb)it.next();
          b.draw(g);
        }
      }
      catch(ConcurrentModificationException e)
      {}
      
      it = explosions.iterator();
      while(it.hasNext())
      {
        CExplosion e = (CExplosion)it.next();
        e.draw(g);
      }
      
      it = doodads.iterator();
      while(it.hasNext())
      {
        CDoodad d = (CDoodad)it.next();
        d.draw(g);
      }
      
      it = planes.iterator();
      while(it.hasNext())
      {
        CPlane p = (CPlane)it.next();
        p.draw(g);
      }
    }
    else
    {
      buyMenu.hide(); // Best place I could think of to put this expression
                      //since I can't put it in the constructor
      g.setColor(Color.BLUE);
      g.setFont(new Font("Verdana", Font.BOLD, 18));
      
      g.drawString("Bunker Command", 430, 270);
      g.drawString("Yet Another Eric Choi Game", 400, 290);
      g.drawString("Version Beta 1 - Released 2005/04/24", 350, 310);
      g.setColor(Color.RED);
      g.drawString("Click to begin!", 440, 330);
    }
  }
  
  public void go()
  {
    while(true)
    {
      if(bunkers.size() != 0) updateThings();
      if(!buyTime && planes.size() == 0 && bombs.size() == 0 && doodads.size() == 0 && bunkers.size() != 0)
      {
        CBunker bunker = getBunker();
        buyTime = true;
        bunker.addHealth(10);
        bunker.refillEnergy();
        removeKeyListener(this);    // Disable keyboard during buy time
        kLeft = kRight = kUp = kDown = kSpace = kShift = false;
        buyMenu.display();
      }
      try{Thread.sleep(30);} catch (InterruptedException e) {}
    }
  }
  
  
  public void keyPressed(KeyEvent e)
  {
    int keyCode = e.getKeyCode();
    switch(keyCode)
    {
      case KeyEvent.VK_LEFT:
        kLeft = true;
        break;
      case KeyEvent.VK_RIGHT:
        kRight = true;
        break;
      case KeyEvent.VK_SPACE:
        kSpace = true;
        break;
      case KeyEvent.VK_UP:
        kUp = true;
        break;
      case KeyEvent.VK_DOWN:
        kDown = true;
        break;
      case KeyEvent.VK_SHIFT:
        kShift = true;
        break;
    }
  }
  
  public void keyTyped(KeyEvent e)
  {
    int keyCode = e.getKeyChar();
    CBunker bunker = getBunker();
    switch(keyCode)
    {
      case KeyEvent.VK_1:
        bunker.setWeapon(CBunker.MGUN_WEAPON);
        break;
      case KeyEvent.VK_2:
        bunker.setWeapon(CBunker.ROCKET_WEAPON);
        break;
      case KeyEvent.VK_3:
        bunker.setWeapon(CBunker.FLAK_WEAPON);
        break;
      case KeyEvent.VK_4:
        bunker.setWeapon(CBunker.LASER_WEAPON);
        break;
    }
  }
  
  public void keyReleased(KeyEvent e)
  {
    int keyCode = e.getKeyCode();
    switch(keyCode)
    {
      case KeyEvent.VK_LEFT:
        kLeft = false;
        break;
      case KeyEvent.VK_RIGHT:
        kRight = false;
        break;
      case KeyEvent.VK_SPACE:
        kSpace = false;
        break;
      case KeyEvent.VK_UP:
        kUp = false;
        break;
      case KeyEvent.VK_DOWN:
        kDown = false;
        break;
      case KeyEvent.VK_SHIFT:
        kShift = false;
        break;
    }
  }
  
  public void checkHit()
  {
    // Check explosions from chaff hitting planes and bombs
    Iterator it = explosions.iterator();
    CBunker bunker = getBunker();
    while(it.hasNext())
    {
      CExplosion e = (CExplosion)it.next();
      
      Iterator planeIt = planes.iterator();
      while(planeIt.hasNext())
      {
        CPlane p = (CPlane)planeIt.next();
        Rectangle r = p.getRect();
        if((distance(new Vector(r.x, r.y), e.getPos()) < e.getRadius()) ||
           (distance(new Vector(r.x+r.width, r.y), e.getPos()) < e.getRadius()) ||
           (distance(new Vector(r.x, r.y+r.height), e.getPos()) < e.getRadius()) ||
           (distance(new Vector(r.x+r.width, r.y+r.height), e.getPos()) < e.getRadius()))
        {
          p.takeDamage(3);
        }
        r = bunker.getRect();
        if((distance(new Vector(r.x, r.y), e.getPos()) < e.getRadius()) ||
           (distance(new Vector(r.x+r.width, r.y), e.getPos()) < e.getRadius()) ||
           (distance(new Vector(r.x, r.y+r.height), e.getPos()) < e.getRadius()) ||
           (distance(new Vector(r.x+r.width, r.y+r.height), e.getPos()) < e.getRadius()))
        {
          bunker.takeDamage(1);
        }        
      }
      
      Iterator bombIt = bombs.iterator();
      while(bombIt.hasNext())
      {
        CBomb b = (CBomb)bombIt.next();
        if(distance(b.getPos(), e.getPos()) < e.getRadius() + b.WIDTH/2)
        {
          b.takeDamage(3);
        } 
      }
    }
    
    // Check for machine gun, rockets and lasers hitting bombs and planes
    it = bullets.iterator();
    
    while(it.hasNext())
    {
      CAmmo b = (CAmmo)it.next();
      
      Iterator planeIt = planes.iterator();
      while(planeIt.hasNext())
      {
        CPlane p = (CPlane)planeIt.next();
        Rectangle r = p.getRect();
        
        Vector pos = b.getPos();
        Vector vel = b.getVel();
        if(r.intersectsLine(pos.x(), pos.y(), pos.x()+vel.x(), pos.y()+vel.y()))
        {
          if(b instanceof CBullet)
            p.takeDamage(7);
          else if(b instanceof CRocket)
            p.takeDamage(10);
          else if(b instanceof CLaser)
            p.takeDamage(10);
          else if(b instanceof CChaff)
            p.takeDamage(5);
          doodads.add(new CDoodad(this, b.getPos(), new Vector(), new Vector()));
          b.die();
        }
      }
      
      Iterator bombIt = bombs.iterator();
      while(bombIt.hasNext())
      {
        CBomb p = (CBomb)bombIt.next();
        Rectangle r = p.getRect();
        
        Vector pos = b.getPos();
        Vector vel = b.getVel();
        if(r.intersectsLine(pos.x(), pos.y(), pos.x()+vel.x(), pos.y()+vel.y()))
        {
          if(b instanceof CBullet)
            p.takeDamage(7);
          else if(b instanceof CRocket)
            p.takeDamage(10);
          else if(b instanceof CLaser)
            p.takeDamage(10);
          else if(b instanceof CChaff)
            p.takeDamage(5);
          bunker.addScore(15);
          bunker.addMoney(15);
          doodads.add(new CDoodad(this, b.getPos(), new Vector(), new Vector()));
          if(!(b instanceof CLaser)) b.die();
        }
      }
    }
    
    // Check for bombs hitting bunker
    Iterator bombIt = bombs.iterator();
    while(bombIt.hasNext())
    {
      CBomb b = (CBomb)bombIt.next();
      
      if(b.getRect().intersects(bunker.getRect()))
      {
        bunker.takeDamage(5);
        doodads.add(new CDoodad(this, b.getPos(), new Vector(), new Vector()));
        b.die();
        if(bunker.isDead()) return;
      }
    }
  }
  
  public double distance(Vector v1, Vector v2)
  {
    double x = v1.x()-v2.x();
    double y = v1.y()-v2.y();
    
    return Math.sqrt(x*x+y*y);
  }
  
  public void actionPerformed(ActionEvent e)
  {
    String command = e.getActionCommand();
    CBunker bunker = getBunker();
          
    if(command.equals("OK"))
    {
      level++;
      buyMenu.hide();
      addKeyListener(this);
      requestFocusInWindow();
      initLevel(level);
      buyTime = false;
    }
    else if(command.equals("Rocket Launcher") && bunker.getMoney() >= 5000)
    {
      if(bunker.buyWeapon(CBunker.ROCKET_WEAPON))
      {
        bunker.addMoney(-5000);
        buyMenu.setRocketLauncherBought();
      }
    }
    else if(command.equals("Buy rockets") && bunker.getMoney() >= 1000)
    {
      bunker.addRockets(50);
      bunker.addMoney(-1000);
    }
    else if(command.equals("Buy mortars") && bunker.getMoney() >= 1500)
    {
      bunker.addMortars(50);
      bunker.addMoney(-1500);
    }
    else if(command.equals("Flak Launcher") && bunker.getMoney () >= 7500)
    {
      if(bunker.buyWeapon(CBunker.FLAK_WEAPON))
      {
        bunker.addMoney(-7500);
        buyMenu.setFlakLauncherBought();
      }
    }
    else if(command.equals("Laser Cannon") && bunker.getMoney() >= 15000)
    {
      if(bunker.buyWeapon(CBunker.LASER_WEAPON))
        bunker.addMoney(-15000);
    }
    else if(command.equals("Restore health") && bunker.getMoney() >= 1500 && bunker.getHealth() < 100)
    {
      bunker.restoreHealth();
      bunker.addMoney(-1500);
    }
  }
  
  public void mousePressed(MouseEvent m)
  {
    if(!buyTime && m.getButton() == MouseEvent.BUTTON1)
    {
      kSpace = true;
    }
    if(!buyTime && m.getButton() == MouseEvent.BUTTON3)
    {
      kShift = true;
    }
  }
  
  public void mouseReleased(MouseEvent m)
  {
    if(m.getButton() == MouseEvent.BUTTON1)
    {
      kSpace = false;
    }
    if(m.getButton() == MouseEvent.BUTTON3)
    {
      kShift = false;
    }
  }
  public void mouseEntered(MouseEvent m) {}
  public void mouseExited(MouseEvent m) {}
  public void mouseClicked(MouseEvent m)
  {
    if(bunkers.size() == 0)
    {
      newGame();
    }
  }
  
  public void mouseMoved(MouseEvent m)
  {
    int x = m.getX();
    int y = m.getY();
    if(bunkers.size() == 0) return;
    CBunker bunker = getBunker();
    Vector v = new Vector((double)x-(double)getWidth()/2.0, (double)getHeight()-(double)y);
    v.scale(0.1);
    bunker.setAim(v);
  }
  
  public void mouseDragged(MouseEvent m) { mouseMoved(m); }
  
  /**
   * Gets an image in the current working directory.
   * 
   * @param fileName File name of the image.
   * */
  Image getImage(String fileName)
  {
    URL imageURL = getClass().getClassLoader().getResource(fileName);
    if(imageURL == null) { return null; }
    return new ImageIcon(imageURL).getImage();
  }
  
  public CBunker getBunker()
  {
    return (CBunker)bunkers.get(0);
  }
}